//go:generate mockgen -package=mockrepository -destination=mock/repository.go . Repository

// Package model provides structures for persistent repository
package model

import (
	"context"
)

// Repository provides persistent operations for auth service
type Repository interface {
	Migrate(ctx context.Context) error
	ResetAttemptInsert(ctx context.Context, attempt *ResetAttempt) error
	ResetAttemptRemoveByRemote(ctx context.Context, remote string) error
	ResetAttemptRemoveAll(ctx context.Context) error
	ResetAttemptGetByRemoteData(ctx context.Context, remote, data string) (*ResetAttempt, error)
	AuthAttemptInsert(ctx context.Context, attempt *AuthAttempt) error
	AuthAttemptRemoveByRemote(ctx context.Context, remote string) error
	AuthAttemptRemoveAll(ctx context.Context) error
	AuthAttemptCountByRemote(ctx context.Context, remote string) (int, error)
	UserAttrGetByNameValue(ctx context.Context, key, value string) ([]string, error)
	UserAttrGetByUserID(ctx context.Context, userID string) ([]*UserAttr, error)
	RecordUserLogin(ctx context.Context, userID, remote string) (known, first bool, err error)
}
