package service

import (
	"context"
	"encoding/json"
	"fmt"

	messaging "gitlab.mashroom.com/mashroom/backend/messaging/api"

	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"

	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
)

func (srv *serverImpl) notifyPrimaryEmailChanged(ctx context.Context, newEmail, oldEmail string) error {
	switch {
	case newEmail == "":
		return fmt.Errorf("%w: new email address is empty", grpcerr.InvalidData)
	case oldEmail == "":
		return fmt.Errorf("%w: old email address is empty", grpcerr.InvalidData)
	}

	bs, _ := json.Marshal(map[string]interface{}{
		"new_email": newEmail,
	})

	_, err := srv.Connections.Notification.SendNotification(ctx, &notification.SendNotificationRequest{
		Entity: notification.Entity_ENTITY_USER_EMAIL,
		Action: notification.Action_ACTION_UPDATE,
		Targets: []*notification.Target{
			{
				Role: notification.Role_ROLE_AUTHOR,
				Contacts: &notification.Contacts{
					Email: oldEmail,
				},
			},
		},
		Embeds: []*notification.EventEmbed{
			{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_JSON,
				EmbedValue: string(bs),
			},
		},
	})

	return err
}

func (srv *serverImpl) notifyPrimaryPhoneChanged(ctx context.Context, newPhone, oldPhone, email string) error {
	switch {
	case newPhone == "":
		return fmt.Errorf("%w: new phone number is empty", grpcerr.InvalidData)
	case oldPhone == "":
		return fmt.Errorf("%w: old phone number is empty", grpcerr.InvalidData)
	case email == "":
		return fmt.Errorf("%w: email address is empty", grpcerr.InvalidData)
	}

	bs, _ := json.Marshal(map[string]interface{}{
		"new_phone": newPhone,
	})

	_, err := srv.Connections.Notification.SendNotification(ctx, &notification.SendNotificationRequest{
		Entity: notification.Entity_ENTITY_USER_PHONE,
		Action: notification.Action_ACTION_UPDATE,
		Targets: []*notification.Target{
			{
				Role: notification.Role_ROLE_AUTHOR,
				Contacts: &notification.Contacts{
					Phone: oldPhone,
				},
			},
		},
		Embeds: []*notification.EventEmbed{
			{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_JSON,
				EmbedValue: string(bs),
			},
		},
	})

	return err
}
