package test

import (
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
)

func (s *serviceSuite) TestAuthAuth_ok() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.account.EXPECT().Get(s.testCtx, &account.GetRequest{
		Id:    s.expectUserID(),
		Admin: false,
		Soft:  false,
	}).Return(&account.GetResponse{
		Account: &account.Account{
			Id: &account.Account_IdValue{IdValue: s.expectUserID()},
		},
	}, nil)

	resp, err := s.srv.Auth(s.testCtx, &auth.AuthRequest{
		AccessToken: s.expectAccessToken(0),
		AdminSite:   false,
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)
}

func (s *serviceSuite) TestAuthAuth_error() {
	s.storage.EXPECT().SessionIntrospect(s.testCtx, s.expectAccessToken(0)).Return(nil, s.expectError())

	var resp, err = s.srv.Auth(s.testCtx, &auth.AuthRequest{
		AccessToken: s.expectAccessToken(0),
		AdminSite:   false,
	})

	s.Require().Error(err)

	s.Require().Nil(resp)
}

func (s *serviceSuite) TestAuthAuth_error_inactive() {
	s.expectSessionIntrospect(s.expectAccessToken(0), false)

	resp, err := s.srv.Auth(s.testCtx, &auth.AuthRequest{
		AccessToken: s.expectAccessToken(0),
		AdminSite:   false,
	})

	s.Require().Error(err)

	s.Require().Nil(resp)
}
