package test

import (
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/storage"
)

func (s *serviceSuite) TestAuthBlock_ok() {
	s.storage.EXPECT().UserGetByID(s.testCtx, s.expectUserID()).Return(&storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}, nil)

	s.storage.EXPECT().UserUpdate(s.testCtx, &storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    false,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}).Return(nil)

	s.account.EXPECT().Block(s.testCtx, &account.BlockRequest{
		Id:    s.expectUserID(),
		Block: true,
	}).Return(&account.BlockResponse{}, nil)

	resp, err := s.srv.BlockUser(s.testCtx, &auth.BlockUserRequest{
		Id:    s.expectUserID(),
		Block: true,
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)
}

func (s *serviceSuite) TestAuthBlock_error() {
	s.storage.EXPECT().UserGetByID(s.testCtx, s.expectUserID()).Return(&storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}, nil)

	s.storage.EXPECT().UserUpdate(s.testCtx, &storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    false,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}).Return(nil)

	s.account.EXPECT().Block(s.testCtx, &account.BlockRequest{
		Id:    s.expectUserID(),
		Block: true,
	}).Return(nil, s.expectError())

	resp, err := s.srv.BlockUser(s.testCtx, &auth.BlockUserRequest{
		Id:    s.expectUserID(),
		Block: true,
	})

	s.Require().Error(err)

	s.Require().Nil(resp)
}
