package test

import (
	"errors"
	"strconv"
	"strings"

	"gitlab.mashroom.com/mashroom/backend/auth/internal/storage"
	"gitlab.mashroom.com/mashroom/backend/auth/keycloak"
)

func (s *serviceSuite) expectUserID() string {
	return "00000000-0000-0000-0000-000000000001"
}

func (s *serviceSuite) expectUserEmail() string {
	return "abc@abc.abc"
}

func (s *serviceSuite) expectUserRemote() string {
	return "1.1.1.1"
}

func (s *serviceSuite) expectError() error {
	return errors.New("someerr")
}

func (s *serviceSuite) expectAccessToken(idx int) string {
	return "access" + strconv.Itoa(idx)
}

func (s *serviceSuite) expectRefreshToken(idx int) string {
	return "refresh" + strconv.Itoa(idx)
}

func (s *serviceSuite) expectSessionState(idx int) string {
	return "sessionstate" + strconv.Itoa(idx)
}

func (s *serviceSuite) expectSessionIntrospect(token string, actve bool, roles ...string) {
	idx := strings.TrimPrefix(strings.TrimPrefix(token, "access"), "refresh")

	s.storage.EXPECT().SessionIntrospect(s.testCtx, token).Return(&storage.SessionIntrospection{
		OpenIDErrorRepresentation: keycloak.OpenIDErrorRepresentation{
			Error:            "",
			ErrorDescription: "",
		},
		OpenIDIntrospectionRepresentation: keycloak.OpenIDIntrospectionRepresentation{
			Jti:               "qqq",
			Exp:               0,
			Nbf:               0,
			Iat:               0,
			Iss:               "",
			Sub:               s.expectUserID(),
			Typ:               "",
			Azp:               "",
			AuthTime:          0,
			SessionState:      "somestate" + idx,
			PreferredUsername: "",
			Email:             s.expectUserEmail(),
			EmailVerified:     false,
			PhoneNumber:       "",
			PhoneVerified:     false,
			Acr:               "",
			AllowedOrigins:    nil,
			Scope:             "",
			ClientID:          "",
			Username:          "",
			Active:            actve,
			RealmAccess: struct {
				Roles []string `json:"roles"`
			}{
				Roles: roles,
			},
		},
	}, nil)
}

func (s *serviceSuite) expectSessionAuth(idx int) *storage.SessionAuth {
	return &storage.SessionAuth{
		OpenIDErrorRepresentation: keycloak.OpenIDErrorRepresentation{
			Error:            "",
			ErrorDescription: "",
		},
		OpenIDAuthRepresentation: keycloak.OpenIDAuthRepresentation{
			AccessToken:      s.expectAccessToken(idx),
			ExpiresIn:        1000,
			RefreshToken:     s.expectRefreshToken(idx),
			RefreshExpiresIn: 1000,
			TokenType:        "",
			NotBeforePolicy:  0,
			SessionState:     s.expectSessionState(idx),
			Scope:            "",
		},
	}
}
