package test

import (
	"time"

	"github.com/golang/mock/gomock"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"

	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/model"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/storage"
)

func (s *serviceSuite) TestAuthForgotPassword_ok() {
	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return([]*storage.User{
		{
			Access:                     nil,
			Attributes:                 nil,
			ClientConsents:             nil,
			ClientRoles:                nil,
			CreatedTimestamp:           0,
			Credentials:                nil,
			DisableableCredentialTypes: nil,
			Email:                      s.expectUserEmail(),
			EmailVerified:              false,
			Enabled:                    true,
			FederatedIdentities:        nil,
			FederationLink:             "",
			FirstName:                  "",
			Groups:                     nil,
			Id:                         s.expectUserID(),
			LastName:                   "",
			NotBefore:                  0,
			Origin:                     "",
			RealmRoles:                 nil,
			RequiredActions:            nil,
			Self:                       "",
			ServiceAccountClientId:     "",
			Username:                   s.expectUserEmail(),
		},
	}, nil)

	s.account.EXPECT().Get(s.testCtx, &account.GetRequest{
		Id: s.expectUserID(),
	}).Return(&account.GetResponse{
		Account: &account.Account{
			Id:        &account.Account_IdValue{IdValue: s.expectUserID()},
			CreatedAt: nil,
			Details:   nil,
			Location:  nil,
			Contact: &account.Account_ContactValue{
				ContactValue: &account.AccountContact{
					EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
					PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
					PhonesSecondary: nil,
					ContactStatus:   nil,
				},
			},
			Favorites:           nil,
			Admin:               nil,
			Block:               nil,
			TosVersion:          nil,
			LastLogin:           nil,
			Deleted:             nil,
			Log:                 nil,
			Test:                nil,
			BankDetails:         nil,
			Resources:           nil,
			CreationType:        nil,
			ViewedVideosIds:     nil,
			EnableLandlordTool:  nil,
			Entrypoint:          0,
			AuthorizationOrigin: 0,
		},
	}, nil)

	s.repo.EXPECT().ResetAttemptGetByRemoteData(
		s.testCtx,
		s.expectUserRemote(),
		"7678bb4af5233f850c8a05d477c3c5c2e1226573bde66cebab2e3722a7274b29",
	).Return(&model.ResetAttempt{
		Time:   time.Now().Add(-time.Hour),
		Remote: s.expectUserRemote(),
		Data:   "7678bb4af5233f850c8a05d477c3c5c2e1226573bde66cebab2e3722a7274b29",
		ID:     1,
	}, nil)

	s.repo.EXPECT().ResetAttemptInsert(s.testCtx, &model.ResetAttempt{
		Remote: s.expectUserRemote(),
		Data:   "7678bb4af5233f850c8a05d477c3c5c2e1226573bde66cebab2e3722a7274b29",
	}).Return(nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    s.expectUserEmail(),
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
			Key:    s.expectUserEmail(),
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "12345678",
		},
	}, nil)

	s.notification.EXPECT().SendNotification(s.testCtx, gomock.Any()).Return(
		&notification.SendNotificationResponse{},
		nil,
	)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    s.expectUserEmail(),
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "12345678",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.storage.EXPECT().UserPasswordReset(s.testCtx, s.expectUserID(), "kkkkkkkk").Return(nil)

	s.storage.EXPECT().UserGetByID(s.testCtx, s.expectUserID()).Return(&storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    false,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}, nil)

	s.storage.EXPECT().UserUpdate(s.testCtx, &storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}).Return(nil)

	s.account.EXPECT().Block(s.testCtx, &account.BlockRequest{
		Id:    s.expectUserID(),
		Block: false,
	}).Return(&account.BlockResponse{}, nil)

	s.repo.EXPECT().ResetAttemptRemoveByRemote(s.testCtx, s.expectUserRemote()).Return(nil)

	s.storage.EXPECT().SessionRemoveAll(s.testCtx, s.expectUserID()).Return(nil)

	s.storage.EXPECT().SessionImpersonate(s.testCtx, s.expectUserID()).Return(s.expectSessionAuth(1), nil)

	resp, err := s.srv.ForgotPassword(s.testCtx, &auth.ForgotPasswordRequest{
		Email:       &auth.ForgotPasswordRequest_EmailValue{EmailValue: s.expectUserEmail()},
		Remote:      &auth.ForgotPasswordRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		Captcha:     nil,
		EmailOtp:    &auth.ForgotPasswordRequest_EmailOtpValue{EmailOtpValue: "12345678"},
		PhoneOtp:    nil,
		NewPassword: &auth.ForgotPasswordRequest_NewPasswordValue{NewPasswordValue: "kkkkkkkk"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Empty(resp.GetErrors())
}

func (s *serviceSuite) TestAuthForgotPassword_error_password() {
	s.storage.EXPECT().UserGetByEmail(s.testCtx, s.expectUserEmail()).Return([]*storage.User{
		{
			Access:                     nil,
			Attributes:                 nil,
			ClientConsents:             nil,
			ClientRoles:                nil,
			CreatedTimestamp:           0,
			Credentials:                nil,
			DisableableCredentialTypes: nil,
			Email:                      s.expectUserEmail(),
			EmailVerified:              false,
			Enabled:                    true,
			FederatedIdentities:        nil,
			FederationLink:             "",
			FirstName:                  "",
			Groups:                     nil,
			Id:                         s.expectUserID(),
			LastName:                   "",
			NotBefore:                  0,
			Origin:                     "",
			RealmRoles:                 nil,
			RequiredActions:            nil,
			Self:                       "",
			ServiceAccountClientId:     "",
			Username:                   s.expectUserEmail(),
		},
	}, nil)

	s.account.EXPECT().Get(s.testCtx, &account.GetRequest{
		Id: s.expectUserID(),
	}).Return(&account.GetResponse{
		Account: &account.Account{
			Id:        &account.Account_IdValue{IdValue: s.expectUserID()},
			CreatedAt: nil,
			Details:   nil,
			Location:  nil,
			Contact: &account.Account_ContactValue{
				ContactValue: &account.AccountContact{
					EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
					PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
					PhonesSecondary: nil,
					ContactStatus:   nil,
				},
			},
			Favorites:           nil,
			Admin:               nil,
			Block:               nil,
			TosVersion:          nil,
			LastLogin:           nil,
			Deleted:             nil,
			Log:                 nil,
			Test:                nil,
			BankDetails:         nil,
			Resources:           nil,
			CreationType:        nil,
			ViewedVideosIds:     nil,
			EnableLandlordTool:  nil,
			Entrypoint:          0,
			AuthorizationOrigin: 0,
		},
	}, nil)

	s.repo.EXPECT().ResetAttemptGetByRemoteData(
		s.testCtx,
		s.expectUserRemote(),
		"50585345cd98ec9a25b626915f28f386f814a3de7be66f46cfaf8b3e1e2ba90f",
	).Return(&model.ResetAttempt{
		Time:   time.Now().Add(-time.Hour),
		Remote: s.expectUserRemote(),
		Data:   "50585345cd98ec9a25b626915f28f386f814a3de7be66f46cfaf8b3e1e2ba90f",
		ID:     1,
	}, nil)

	s.repo.EXPECT().ResetAttemptInsert(s.testCtx, &model.ResetAttempt{
		Remote: s.expectUserRemote(),
		Data:   "50585345cd98ec9a25b626915f28f386f814a3de7be66f46cfaf8b3e1e2ba90f",
	}).Return(nil)

	s.invitation.EXPECT().ContactValidationCreate(s.testCtx, &invitation.ContactValidationCreateRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    s.expectUserEmail(),
		UserId: s.expectUserID(),
		Remote: "",
		Force:  false,
	}).Return(&invitation.ContactValidationCreateResponse{
		ContactValidation: &invitation.ContactValidation{
			Id:     "1",
			Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
			Key:    s.expectUserEmail(),
			UserId: s.expectUserID(),
			Remote: "",
			Otp:    "12345678",
		},
	}, nil)

	s.notification.EXPECT().SendNotification(s.testCtx, gomock.Any()).Return(
		&notification.SendNotificationResponse{},
		nil,
	)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    s.expectUserEmail(),
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "12345678",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	resp, err := s.srv.ForgotPassword(s.testCtx, &auth.ForgotPasswordRequest{
		Email:       &auth.ForgotPasswordRequest_EmailValue{EmailValue: s.expectUserEmail()},
		Remote:      &auth.ForgotPasswordRequest_RemoteValue{RemoteValue: s.expectUserRemote()},
		Captcha:     nil,
		EmailOtp:    &auth.ForgotPasswordRequest_EmailOtpValue{EmailOtpValue: "12345678"},
		PhoneOtp:    nil,
		NewPassword: &auth.ForgotPasswordRequest_NewPasswordValue{NewPasswordValue: "kkk"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_PASSWORD_TOO_SHORT)
}
