package test

import (
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/storage"
)

func (s *serviceSuite) TestAuthServiceUpdatePassword_ok() {
	s.account.EXPECT().Get(s.testCtx, &account.GetRequest{
		Id: s.expectUserID(),
	}).Return(&account.GetResponse{
		Account: &account.Account{
			Id:        &account.Account_IdValue{IdValue: s.expectUserID()},
			CreatedAt: nil,
			Details:   nil,
			Location:  nil,
			Contact: &account.Account_ContactValue{
				ContactValue: &account.AccountContact{
					EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
					PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
					PhonesSecondary: nil,
					ContactStatus:   nil,
				},
			},
			Favorites:           nil,
			Admin:               nil,
			Block:               nil,
			TosVersion:          nil,
			LastLogin:           nil,
			Deleted:             nil,
			Log:                 nil,
			Test:                nil,
			BankDetails:         nil,
			Resources:           nil,
			CreationType:        nil,
			ViewedVideosIds:     nil,
			EnableLandlordTool:  nil,
			Entrypoint:          0,
			AuthorizationOrigin: 0,
		},
	}, nil)

	s.storage.EXPECT().UserPasswordReset(s.testCtx, s.expectUserID(), "kkkkkkkk").Return(nil)

	resp, err := s.srv.ServiceUpdatePassword(s.testCtx, &auth.ServiceUpdatePasswordRequest{
		UserId:      s.expectUserID(),
		NewPassword: "kkkkkkkk",
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Empty(resp.GetErrors())
}

func (s *serviceSuite) TestAuthServiceUpdatePassword_ok_blocked() {
	acc := &account.Account{
		Id:        &account.Account_IdValue{IdValue: s.expectUserID()},
		CreatedAt: nil,
		Details:   nil,
		Location:  nil,
		Contact: &account.Account_ContactValue{
			ContactValue: &account.AccountContact{
				EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
				PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
				PhonesSecondary: nil,
				ContactStatus:   nil,
			},
		},
		Favorites:           nil,
		Admin:               nil,
		Block:               &account.Account_BlockValue{BlockValue: true},
		TosVersion:          nil,
		LastLogin:           nil,
		Deleted:             nil,
		Log:                 nil,
		Test:                nil,
		BankDetails:         nil,
		Resources:           nil,
		CreationType:        nil,
		ViewedVideosIds:     nil,
		EnableLandlordTool:  nil,
		Entrypoint:          0,
		AuthorizationOrigin: 0,
	}

	s.account.EXPECT().Get(s.testCtx, &account.GetRequest{
		Id: s.expectUserID(),
	}).Return(&account.GetResponse{
		Account: acc,
	}, nil)

	s.storage.EXPECT().UserPasswordReset(s.testCtx, s.expectUserID(), "kkkkkkkk").Return(nil)

	s.storage.EXPECT().UserGetByID(s.testCtx, s.expectUserID()).Return(&storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    false,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}, nil)

	s.storage.EXPECT().UserUpdate(s.testCtx, &storage.User{
		Access:                     nil,
		Attributes:                 map[string]interface{}{},
		ClientConsents:             nil,
		ClientRoles:                nil,
		CreatedTimestamp:           0,
		Credentials:                nil,
		DisableableCredentialTypes: nil,
		Email:                      s.expectUserEmail(),
		EmailVerified:              false,
		Enabled:                    true,
		FederatedIdentities:        nil,
		FederationLink:             "",
		FirstName:                  "",
		Groups:                     nil,
		Id:                         s.expectUserID(),
		LastName:                   "",
		NotBefore:                  0,
		Origin:                     "",
		RealmRoles:                 nil,
		RequiredActions:            nil,
		Self:                       "",
		ServiceAccountClientId:     "",
		Username:                   s.expectUserEmail(),
	}).Return(nil)

	s.account.EXPECT().Block(s.testCtx, &account.BlockRequest{
		Id:    s.expectUserID(),
		Block: false,
	}).Return(&account.BlockResponse{}, nil)

	resp, err := s.srv.ServiceUpdatePassword(s.testCtx, &auth.ServiceUpdatePasswordRequest{
		UserId:      s.expectUserID(),
		NewPassword: "kkkkkkkk",
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Empty(resp.GetErrors())
}

func (s *serviceSuite) TestAuthServiceUpdatePassword_error_password() {
	resp, err := s.srv.ServiceUpdatePassword(s.testCtx, &auth.ServiceUpdatePasswordRequest{
		UserId:      s.expectUserID(),
		NewPassword: "kkk",
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_PASSWORD_TOO_SHORT)
}
