package test

import (
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	auth "gitlab.mashroom.com/mashroom/backend/auth/api"
	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"
)

func (s *serviceSuite) TestAuthVerifyEmail_ok() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.account.EXPECT().Get(s.testCtx, &account.GetRequest{
		Id: s.expectUserID(),
	}).Return(&account.GetResponse{
		Account: &account.Account{
			Id:        &account.Account_IdValue{IdValue: s.expectUserID()},
			CreatedAt: nil,
			Details:   nil,
			Location:  nil,
			Contact: &account.Account_ContactValue{
				ContactValue: &account.AccountContact{
					EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
					PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
					PhonesSecondary: nil,
					ContactStatus:   nil,
				},
			},
			Favorites:           nil,
			Admin:               nil,
			Block:               nil,
			TosVersion:          nil,
			LastLogin:           nil,
			Deleted:             nil,
			Log:                 nil,
			Test:                nil,
			BankDetails:         nil,
			Resources:           nil,
			CreationType:        nil,
			ViewedVideosIds:     nil,
			EnableLandlordTool:  nil,
			Entrypoint:          0,
			AuthorizationOrigin: 0,
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    s.expectUserEmail(),
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "12345678",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil)

	s.account.EXPECT().Update(s.testCtx, &account.UpdateRequest{
		Account: &account.Account{
			Id: &account.Account_IdValue{
				IdValue: s.expectUserID(),
			},
			Contact: &account.Account_ContactValue{
				ContactValue: &account.AccountContact{
					ContactStatus: &account.AccountContact_ContactStatusValue{
						ContactStatusValue: &account.AccountContactStatus{
							EmailVerified: &account.AccountContactStatus_EmailVerifiedValue{
								EmailVerifiedValue: true,
							},
						},
					},
				},
			},
		},
	})

	resp, err := s.srv.VerifyEmail(s.testCtx, &auth.VerifyEmailRequest{
		AccessToken: s.expectAccessToken(0),
		EmailOtp:    &auth.VerifyEmailRequest_EmailOtpValue{EmailOtpValue: "12345678"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Empty(resp.GetErrors())
}

func (s *serviceSuite) TestAuthVerifyEmail_error_otp() {
	s.expectSessionIntrospect(s.expectAccessToken(0), true)

	s.account.EXPECT().Get(s.testCtx, &account.GetRequest{
		Id: s.expectUserID(),
	}).Return(&account.GetResponse{
		Account: &account.Account{
			Id:        &account.Account_IdValue{IdValue: s.expectUserID()},
			CreatedAt: nil,
			Details:   nil,
			Location:  nil,
			Contact: &account.Account_ContactValue{
				ContactValue: &account.AccountContact{
					EmailPrimary:    &account.AccountContact_EmailPrimaryValue{EmailPrimaryValue: s.expectUserEmail()},
					PhonePrimary:    &account.AccountContact_PhonePrimaryValue{PhonePrimaryValue: "71002003040"},
					PhonesSecondary: nil,
					ContactStatus:   nil,
				},
			},
			Favorites:           nil,
			Admin:               nil,
			Block:               nil,
			TosVersion:          nil,
			LastLogin:           nil,
			Deleted:             nil,
			Log:                 nil,
			Test:                nil,
			BankDetails:         nil,
			Resources:           nil,
			CreationType:        nil,
			ViewedVideosIds:     nil,
			EnableLandlordTool:  nil,
			Entrypoint:          0,
			AuthorizationOrigin: 0,
		},
	}, nil)

	s.invitation.EXPECT().ContactValidationVerify(s.testCtx, &invitation.ContactValidationVerifyRequest{
		Type:   invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL,
		Key:    s.expectUserEmail(),
		UserId: s.expectUserID(),
		Remote: "",
		Otp:    "12345678",
		Force:  false,
	}).Return(&invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_EXPIRED,
	}, nil)

	resp, err := s.srv.VerifyEmail(s.testCtx, &auth.VerifyEmailRequest{
		AccessToken: s.expectAccessToken(0),
		EmailOtp:    &auth.VerifyEmailRequest_EmailOtpValue{EmailOtpValue: "12345678"},
	})

	s.Require().NoError(err)

	s.Require().NotNil(resp)

	s.Require().Contains(resp.GetErrors(), auth.AuthErrors_AUTH_ERROR_EMAIL_OTP_EXPIRED)
}
