package test

import (
	"context"
	"testing"

	"github.com/golang/mock/gomock"
	"github.com/stretchr/testify/suite"
	mockaccount "gitlab.mashroom.com/mashroom/backend/account/api/mock"
	mockrepository "gitlab.mashroom.com/mashroom/backend/auth/internal/model/mock"
	"gitlab.mashroom.com/mashroom/backend/auth/internal/service"
	mockstorage "gitlab.mashroom.com/mashroom/backend/auth/internal/storage/mock"
	mockinvitation "gitlab.mashroom.com/mashroom/backend/invitation/api/mock"
	mocknotification "gitlab.mashroom.com/mashroom/backend/notification/api/mock"
)

type serviceSuite struct {
	suite.Suite
	srv                    service.Service
	rootCtx, testCtx       context.Context
	rootCancel, testCancel context.CancelFunc
	ctrl                   *gomock.Controller
	repo                   *mockrepository.MockRepository
	storage                *mockstorage.MockRepository
	account                *mockaccount.MockAccountServiceClient
	invitation             *mockinvitation.MockInvitationServiceClient
	notification           *mocknotification.MockNotificationServiceClient
}

func (s *serviceSuite) SetupSuite() {
	s.rootCtx, s.rootCancel = context.WithCancel(context.Background())
}

func (s *serviceSuite) SetupTest() {
	s.ctrl = gomock.NewController(s.T())
	s.repo = mockrepository.NewMockRepository(s.ctrl)
	s.storage = mockstorage.NewMockRepository(s.ctrl)
	s.account = mockaccount.NewMockAccountServiceClient(s.ctrl)
	s.invitation = mockinvitation.NewMockInvitationServiceClient(s.ctrl)
	s.notification = mocknotification.NewMockNotificationServiceClient(s.ctrl)
	s.testCtx, s.testCancel = context.WithCancel(s.rootCtx)

	var err error

	s.srv, err = service.NewService(&service.Config{
		Repo:    s.repo,
		Storage: s.storage,
		Connections: service.Connections{
			Account:      s.account,
			Notification: s.notification,
			Invitation:   s.invitation,
		},
		ReCaptchaKey:  "abc",
		AuthThreshold: 3,
		OAuthEnabled:  false,
	})
	s.Require().NoError(err)
}

func (s *serviceSuite) TearDownTest() {
	s.ctrl.Finish()

	s.testCancel()
}

func (s *serviceSuite) TearSuite() {
	s.rootCancel()
}

func TestService(t *testing.T) {
	suite.Run(t, &serviceSuite{})
}
